#!/bin/sh

PATH=$PATH:/bin:/usr/bin:/usr/local/bin

# gettext package test
if [ ! -x `which xgettext` ]
then
	echo "xgettext was not found. please install gettext packages."
	exit 0;
fi

SCRIPTS=$0
MAILADDR="webmaster at momo-i.org"
OLANG=`echo $LANG | sed 's/\..*//;'`
POTNAME="messages.pot"
PODIR="../$OLANG/LC_MESSAGES"
PONAME="messages.po"
MONAME="messages.mo"

if [ ! -d $PODIR ]
then
	mkdir -p $PODIR
	echo "$PODIR has created."
fi

if [ $# = 0 ]; then
	echo "usage: $SCRIPTS [--help|--get|--init|--merge|--format|--all]"
	exit
fi

case $1 in
	"--all"|"-a"|"all")
		xgettext --add-comments=HINT: --package-name="Cacti I18N" --package-version="0.8.7e" --from-code=UTF-8 --msgid-bugs-address="$MAILADDR" -L php --keyword=_e --keyword=gettext_noop -o $POTNAME `find ../../ -name \*.php -type f` `find ../../ -name \*.inc -type f`
		OLANG=`ls ../ | grep -v base`
		echo "add database words add to pot file..."
		for i in $OLANG
		do
			echo "$i PO file merging..."
			msgmerge --update ../$i/LC_MESSAGES/messages.po messages.pot
			echo "$i MO file creating..."
			msgfmt -v -c ../$i/LC_MESSAGES/messages.po -o ../$i/LC_MESSAGES/messages.mo
			rm -f ../$i/LC_MESSAGES/messages.po~
			obs=`cat ../$i/LC_MESSAGES/messages.po | grep '^#~' | wc -l`
			echo "Obsolete: $obs"
		done
		;;
	"--get"|"-g"|"get")
		xgettext --add-comments=HINT: --package-name="Cacti I18N" --package-version="0.8.7e" --from-code=UTF-8 --msgid-bugs-address="$MAILADDR" -L php --keyword=_e --keyword=gettext_noop -o $POTNAME `find ../../ -name \*.php -type f` `find ../../ -name \*.inc -type f`
		if [ $? = 0 ]; then
			echo "pot file creation was done.";
		else
			echo "pot file creation wasn't done.";
		fi
		echo "add database words to pot file..."
		;;
	"--init"|"-i"|"init")
		msginit -l $LANG -i $POTNAME -o $PODIR/$PONAME;
		;;
	"--format"|"-f"|"format")
		msgfmt -v --check $PODIR/$PONAME -o $PODIR/$MONAME;
		;;
	"--merge"|"-m"|"merge")
		msgmerge --update $PODIR/$PONAME $POTNAME;
		rm -f $PODIR/messages.po~
		;;
	"--help"|"-h"|"help"|"*")
		echo "usage: $SCRIPTS [--help|--get|--init|--merge|--format|--all]";
		echo "";
		;;
esac
